import socket
import subprocess

import click
import requests
import toml
from twilio.base.exceptions import TwilioRestException
from twilio.rest import Client as TwilioClient

HOSTNAME = socket.gethostname()


def get_status_text(unit, user):
    cmd = ["systemctl"]
    if user:
        cmd.append("--user")
    cmd.append("status")
    cmd.append(unit)
    result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    return f"{unit} has failed on {HOSTNAME}\n" + result.stdout.decode()


def send_twilio_message(twilio_config, status_text):
    client = TwilioClient(twilio_config["account"], twilio_config["token"])
    try:
        client.messages.create(
            to=twilio_config["to"], from_=twilio_config["from"], body=status_text
        )
    except TwilioRestException as e:
        print(e)


def send_mailgun_message(mailgun_config, status_text, unit):
    with requests.Session() as session:
        response = session.post(
            mailgun_config["endpoint"],
            auth=("api", mailgun_config["key"]),
            data={
                "from": f"{mailgun_config['from_name']} "
                f"<{mailgun_config['from_address']}>",
                "to": mailgun_config["to_address"],
                "subject": f"{unit} has failed on {HOSTNAME}",
                "text": status_text,
            },
        )
        if response.status_code != 200:
            print("Could not send message.")
            print(response.text)


def handle_providers(providers: dict, status_text, unit):
    if twilio_config := providers.get("twilio"):
        send_twilio_message(twilio_config, status_text)
    if mailgun_config := providers.get("mailgun"):
        send_mailgun_message(mailgun_config, status_text, unit)


@click.command()
@click.argument("unit")
@click.option("--user", is_flag=True)
@click.option(
    "-c",
    "--config-path",
    type=click.Path(exists=True),
    default="/etc/sd-fail-notify/config.toml",
)
def cli(unit, user, config_path):
    status_text = get_status_text(unit, user)
    config = toml.load(config_path)
    handle_providers(config["providers"], status_text, unit)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['sd_fail_notify']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'requests>=2.22,<3.0',
 'toml>=0.10.0,<0.11.0',
 'twilio>=6.33,<7.0']

entry_points = \
{'console_scripts': ['sd-fail-notify = sd_fail_notify.main:cli']}

setup_kwargs = {
    'name': 'sd-fail-notify',
    'version': '19.11.1',
    'description': 'Send a text message when a systemd process fails',
    'long_description': '# Systemd Failure Notifier\nThis is a simple command utility designed to be run on the failure of a systemd unit.\n\n## Usage\n`sd-fail-notify <unit-name>`\n \n It is meant to be used with the `%i` unit name option passed in a systemd OnFailure dependency\n \n ## Supported Providers\n \n ### Twilio\n \n Required configuration: from, to, account, token. See example config for details.\n',
    'author': 'Rick Henry',
    'author_email': 'rickhenry@rickhenry.dev',
    'url': 'https://github.com/rickh94/sd_fail_notify.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
